<?php
ini_set('display_errors', 1);
ini_set('display_startup_errors', 1);
error_reporting(E_ALL);

require_once '../../includes/auth.php';
requireLocationAdmin();

$locationId = getCurrentUserLocationId();
$location = getLocationById($locationId);

$message = '';
$messageType = '';
$customerData = null;

// Process barcode scan
if ($_SERVER['REQUEST_METHOD'] === 'POST' && isset($_POST['barcode'])) {
    $barcode = $_POST['barcode'];
    
    // Get customer data from barcode
    $customerData = getCustomerByBarcode($barcode);
    
    if (!$customerData) {
        $message = "Invalid barcode or customer not found";
        $messageType = "error";
    } else {
        // Check if customer belongs to this location
        if ($customerData['location_id'] != $locationId && $_SESSION['user_role'] !== 'super_admin') {
            $message = "This customer is not registered with your location";
            $messageType = "error";
            $customerData = null;
        }
    }
}

// Process stamp action
if ($_SERVER['REQUEST_METHOD'] === 'POST' && isset($_POST['action']) && isset($_POST['card_id'])) {
    $cardId = $_POST['card_id'];
    $action = $_POST['action'];
    
    if ($action === 'add') {
        if (addStamp($cardId)) {
            $message = "Stamp added successfully";
            $messageType = "success";
            
            // Refresh customer data
            $customerData = getCustomerByCardId($cardId);
        } else {
            $message = "Failed to add stamp";
            $messageType = "error";
        }
    } else if ($action === 'remove') {
        if (removeStamp($cardId)) {
            $message = "Stamp removed successfully";
            $messageType = "success";
            
            // Refresh customer data
            $customerData = getCustomerByCardId($cardId);
        } else {
            $message = "Failed to remove stamp";
            $messageType = "error";
        }
    }
}

// Search and pagination parameters
$search = isset($_GET['search']) ? trim($_GET['search']) : '';
$page = isset($_GET['page']) ? (int)$_GET['page'] : 1;
$perPage = 10;
$offset = ($page - 1) * $perPage;

// Get all card numbers for quick search with search and pagination
$conn = getDbConnection();

// Count total customers for pagination
$countQuery = "
    SELECT COUNT(*) 
    FROM loyalty_cards lc
    JOIN customers c ON lc.customer_id = c.id
    WHERE c.location_id = :location_id
";

if (!empty($search)) {
    $countQuery .= " AND (c.name LIKE :search OR c.email LIKE :search OR lc.card_number LIKE :search)";
}

$stmt = $conn->prepare($countQuery);
$stmt->bindParam(':location_id', $locationId);

if (!empty($search)) {
    $searchParam = "%{$search}%";
    $stmt->bindParam(':search', $searchParam);
}

$stmt->execute();
$totalCards = $stmt->fetchColumn();
$totalPages = ceil($totalCards / $perPage);

// Get cards with pagination
$query = "
    SELECT lc.card_number, c.name, c.email, lc.stamp_count, lc.rewards_earned
    FROM loyalty_cards lc
    JOIN customers c ON lc.customer_id = c.id
    WHERE c.location_id = :location_id
";

if (!empty($search)) {
    $query .= " AND (c.name LIKE :search OR c.email LIKE :search OR lc.card_number LIKE :search)";
}

$query .= " ORDER BY c.name LIMIT :offset, :limit";

$stmt = $conn->prepare($query);
$stmt->bindParam(':location_id', $locationId);

if (!empty($search)) {
    $searchParam = "%{$search}%";
    $stmt->bindParam(':search', $searchParam);
}

$stmt->bindParam(':offset', $offset, PDO::PARAM_INT);
$stmt->bindParam(':limit', $perPage, PDO::PARAM_INT);
$stmt->execute();
$allCards = $stmt->fetchAll(PDO::FETCH_ASSOC);
?>
<!DOCTYPE html>
<html lang="en">
<head>
    <meta charset="UTF-8">
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <title>Scan Barcode - Pointify</title>
    <link href="https://cdn.jsdelivr.net/npm/tailwindcss@2.2.19/dist/tailwind.min.css" rel="stylesheet">
    <link rel="stylesheet" href="https://cdnjs.cloudflare.com/ajax/libs/font-awesome/6.0.0-beta3/css/all.min.css">
    <!-- Add HTML5-QRCode library -->
    <script src="https://unpkg.com/html5-qrcode"></script>
    <style>
        #reader {
            width: 100%;
            max-width: 500px;
            margin: 0 auto;
            border-radius: 0.5rem;
            overflow: hidden;
        }
    </style>
</head>
<body class="bg-gray-100 min-h-screen">
    <!-- Navigation -->
    <nav class="bg-blue-600 text-white p-4">
        <div class="container mx-auto flex justify-between items-center">
            <h1 class="text-xl font-bold">Pointify</h1>
            <div class="flex items-center space-x-4">
                <a href="dashboard.php" class="hover:underline">Dashboard</a>
                <span>Welcome, <?php echo htmlspecialchars($_SESSION['user_name']); ?></span>
                <a href="../logout.php" class="bg-blue-700 px-3 py-1 rounded hover:bg-blue-800">Logout</a>
            </div>
        </div>
    </nav>
    
    <!-- Main Content -->
    <div class="container mx-auto p-4">
        <div class="bg-white rounded-lg shadow-md p-6 mb-6">
            <h2 class="text-2xl font-bold mb-4">Scan Barcode</h2>
            
            <?php if ($message): ?>
                <div class="<?php echo $messageType === 'error' ? 'bg-red-100 border-red-400 text-red-700' : 'bg-green-100 border-green-400 text-green-700'; ?> px-4 py-3 rounded mb-4 border">
                    <?php echo htmlspecialchars($message); ?>
                </div>
            <?php endif; ?>
            
            <form method="post" class="space-y-4" id="barcodeForm">
                <div>
                    <label for="barcode" class="block text-gray-700 mb-2">Enter or Scan Barcode</label>
                    <div class="flex space-x-2">
                        <input type="text" id="barcode" name="barcode" autofocus
                            class="flex-1 px-4 py-2 border rounded-lg focus:outline-none focus:ring-2 focus:ring-blue-500"
                            placeholder="Enter barcode or scan barcode">
                        <button type="button" id="scanButton" class="bg-blue-500 text-white py-2 px-4 rounded-lg hover:bg-blue-600 transition duration-200">
                            <svg xmlns="http://www.w3.org/2000/svg" class="h-5 w-5 inline-block mr-1" viewBox="0 0 20 20" fill="currentColor">
                                <path fill-rule="evenodd" d="M4 5a2 2 0 00-2 2v8a2 2 0 002 2h12a2 2 0 002-2V7a2 2 0 00-2-2h-1.586a1 1 0 01-.707-.293l-1.121-1.121A2 2 0 0011.172 3H8.828a2 2 0 00-1.414.586L6.293 4.707A1 1 0 015.586 5H4zm6 9a3 3 0 100-6 3 3 0 000 6z" clip-rule="evenodd" />
                            </svg>
                            Scan
                        </button>
                    </div>
                </div>
                
                <!-- Scanner Container (hidden by default) -->
                <div id="scanner-container" class="hidden mb-4">
                    <div id="reader"></div>
                    <button type="button" id="closeScanner" class="mt-2 bg-red-500 text-white py-2 px-4 rounded-lg hover:bg-red-600 transition duration-200">
                        Close Scanner
                    </button>
                </div>
                
                <button type="submit" class="bg-blue-500 text-white py-2 px-4 rounded-lg hover:bg-blue-600 transition duration-200">
                    Lookup Customer
                </button>
            </form>
            
            <?php if ($customerData): ?>
                <div class="border-t pt-6 mt-6">
                    <h2 class="text-xl font-semibold mb-4">Customer Information</h2>
                    
                    <div class="grid grid-cols-1 md:grid-cols-2 gap-4 mb-6">
                        <div>
                            <p><strong>Name:</strong> <?php echo htmlspecialchars($customerData['name']); ?></p>
                            <p><strong>Email:</strong> <?php echo htmlspecialchars($customerData['email']); ?></p>
                            <p><strong>Card Number:</strong> <?php echo htmlspecialchars($customerData['card_number']); ?></p>
                        </div>
                        <div>
                            <p><strong>Current Stamps:</strong> <?php echo htmlspecialchars($customerData['stamp_count']); ?></p>
                            <p><strong>Reward Threshold:</strong> <?php echo htmlspecialchars($customerData['reward_threshold']); ?></p>
                            <p><strong>Rewards Earned:</strong> <?php echo htmlspecialchars($customerData['rewards_earned'] ?? 0); ?></p>
                            <p><strong>Progress:</strong> 
                                <?php echo htmlspecialchars($customerData['stamp_count']); ?> / 
                                <?php echo htmlspecialchars($customerData['reward_threshold']); ?>
                                (<?php echo round(($customerData['stamp_count'] / $customerData['reward_threshold']) * 100); ?>%)
                            </p>
                        </div>
                    </div>
                    
                    <div class="bg-gray-50 p-4 rounded-lg mb-6">
                        <h3 class="text-lg font-semibold mb-2">Stamp Actions</h3>
                        <div class="flex space-x-4">
                            <form method="post">
                                <input type="hidden" name="card_id" value="<?php echo htmlspecialchars($customerData['card_id']); ?>">
                                <input type="hidden" name="action" value="add">
                                <button type="submit" class="bg-green-500 text-white py-2 px-4 rounded-lg hover:bg-green-600 transition duration-200">
                                    <i class="fas fa-plus mr-2"></i> Add Stamp
                                </button>
                            </form>
                            
                            <form method="post">
                                <input type="hidden" name="card_id" value="<?php echo htmlspecialchars($customerData['card_id']); ?>">
                                <input type="hidden" name="action" value="remove">
                                <button type="submit" class="bg-red-500 text-white py-2 px-4 rounded-lg hover:bg-red-600 transition duration-200"
                                        <?php echo $customerData['stamp_count'] <= 0 ? 'disabled' : ''; ?>>
                                    <i class="fas fa-minus mr-2"></i> Remove Stamp
                                </button>
                            </form>
                        </div>
                    </div>
                    
                    <!-- Visual representation of stamps -->
                    <div class="mb-6">
                        <h3 class="text-lg font-semibold mb-2">Stamp Card</h3>
                        <div class="grid grid-cols-5 gap-2">
                            <?php for ($i = 1; $i <= $customerData['reward_threshold']; $i++): ?>
                                <div class="aspect-w-1 aspect-h-1">
                                    <div class="<?php echo $i <= $customerData['stamp_count'] ? 'bg-yellow-500' : 'bg-gray-200'; ?> rounded-full flex items-center justify-center text-white font-bold text-lg p-4">
                                        <?php echo $i; ?>
                                    </div>
                                </div>
                            <?php endfor; ?>
                        </div>
                    </div>
                    
                    <?php if ($customerData['stamp_count'] >= $customerData['reward_threshold']): ?>
                        <div class="bg-green-100 border-green-400 text-green-700 px-4 py-3 rounded mb-4 border">
                            <p class="font-bold">Customer has earned a reward!</p>
                            <p>They have collected <?php echo htmlspecialchars($customerData['stamp_count']); ?> stamps.</p>
                            <p class="mt-2">Adding one more stamp will reset the count and record this reward.</p>
                        </div>
                    <?php endif; ?>
                </div>
            <?php endif; ?>
        </div>
        
        <!-- Customer Cards Table -->
        <div class="bg-white rounded-lg shadow-md p-6">
            <h2 class="text-2xl font-bold mb-4">All Cards</h2>
            
            <!-- Search Form -->
            <form method="get" class="mb-6">
                <div class="flex space-x-2">
                    <input type="text" name="search" value="<?php echo htmlspecialchars($search); ?>" 
                           class="flex-1 px-4 py-2 border rounded-lg focus:outline-none focus:ring-2 focus:ring-blue-500"
                           placeholder="Search by name, email or card number">
                    <button type="submit" class="bg-blue-500 text-white py-2 px-4 rounded-lg hover:bg-blue-600 transition duration-200">
                        <i class="fas fa-search mr-2"></i> Search
                    </button>
                    <?php if (!empty($search)): ?>
                        <a href="?page=1" class="bg-gray-500 text-white py-2 px-4 rounded-lg hover:bg-gray-600 transition duration-200">
                            Clear
                        </a>
                    <?php endif; ?>
                </div>
            </form>
            
            <div class="overflow-x-auto">
                <table class="min-w-full bg-white">
                    <thead class="bg-gray-100">
                        <tr>
                            <th class="py-2 px-4 text-left">Customer</th>
                            <th class="py-2 px-4 text-left">Email</th>
                            <th class="py-2 px-4 text-left">Card Number</th>
                            <th class="py-2 px-4 text-left">Stamps</th>
                            <th class="py-2 px-4 text-left">Rewards</th>
                            <th class="py-2 px-4 text-left">Action</th>
                        </tr>
                    </thead>
                    <tbody>
                        <?php if (count($allCards) > 0): ?>
                            <?php foreach ($allCards as $card): ?>
                                <tr class="border-t hover:bg-gray-50">
                                    <td class="py-2 px-4"><?php echo htmlspecialchars($card['name']); ?></td>
                                    <td class="py-2 px-4"><?php echo htmlspecialchars($card['email']); ?></td>
                                    <td class="py-2 px-4"><?php echo htmlspecialchars($card['card_number']); ?></td>
                                    <td class="py-2 px-4"><?php echo htmlspecialchars($card['stamp_count']); ?></td>
                                    <td class="py-2 px-4"><?php echo htmlspecialchars($card['rewards_earned'] ?? 0); ?></td>
                                    <td class="py-2 px-4">
                                        <button type="button" class="select-card-btn bg-blue-500 text-white py-1 px-2 rounded hover:bg-blue-600 transition duration-200" 
                                                data-card="<?php echo htmlspecialchars($card['card_number']); ?>">
                                            Select
                                        </button>
                                    </td>
                                </tr>
                            <?php endforeach; ?>
                        <?php else: ?>
                            <tr>
                                <td colspan="6" class="py-4 px-4 text-center text-gray-500">No cards found</td>
                            </tr>
                        <?php endif; ?>
                    </tbody>
                </table>
            </div>
            
            <!-- Pagination -->
            <?php if ($totalPages > 1): ?>
                <div class="mt-6 flex justify-center">
                    <div class="flex space-x-1">
                        <?php if ($page > 1): ?>
                            <a href="?page=<?php echo $page - 1; ?><?php echo !empty($search) ? '&search=' . urlencode($search) : ''; ?>" 
                               class="px-4 py-2 bg-gray-200 text-gray-800 rounded hover:bg-gray-300">
                                Previous
                            </a>
                        <?php endif; ?>
                        
                        <?php
                        // Show a limited number of page links
                        $startPage = max(1, $page - 2);
                        $endPage = min($totalPages, $page + 2);
                        
                        // Always show first page
                        if ($startPage > 1) {
                            echo '<a href="?page=1' . (!empty($search) ? '&search=' . urlencode($search) : '') . '" 
                                     class="px-4 py-2 bg-gray-200 text-gray-800 rounded hover:bg-gray-300">1</a>';
                            if ($startPage > 2) {
                                echo '<span class="px-4 py-2">...</span>';
                            }
                        }
                        
                        // Show page links
                        for ($i = $startPage; $i <= $endPage; $i++) {
                            $activeClass = $i === $page ? 'bg-blue-500 text-white' : 'bg-gray-200 text-gray-800 hover:bg-gray-300';
                            echo '<a href="?page=' . $i . (!empty($search) ? '&search=' . urlencode($search) : '') . '" 
                                     class="px-4 py-2 ' . $activeClass . ' rounded">' . $i . '</a>';
                        }
                        
                        // Always show last page
                        if ($endPage < $totalPages) {
                            if ($endPage < $totalPages - 1) {
                                echo '<span class="px-4 py-2">...</span>';
                            }
                            echo '<a href="?page=' . $totalPages . (!empty($search) ? '&search=' . urlencode($search) : '') . '" 
                                     class="px-4 py-2 bg-gray-200 text-gray-800 rounded hover:bg-gray-300">' . $totalPages . '</a>';
                        }
                        ?>
                        
                        <?php if ($page < $totalPages): ?>
                            <a href="?page=<?php echo $page + 1; ?><?php echo !empty($search) ? '&search=' . urlencode($search) : ''; ?>" 
                               class="px-4 py-2 bg-gray-200 text-gray-800 rounded hover:bg-gray-300">
                                Next
                            </a>
                        <?php endif; ?>
                    </div>
                </div>
            <?php endif; ?>
            
            <div class="mt-4 text-center text-sm text-gray-600">
                Showing <?php echo count($allCards); ?> of <?php echo $totalCards; ?> cards
            </div>
        </div>
    </div>
    
    <script>
        document.addEventListener('DOMContentLoaded', function() {
            const scanButton = document.getElementById('scanButton');
            const closeScanner = document.getElementById('closeScanner');
            const scannerContainer = document.getElementById('scanner-container');
            const barcodeInput = document.getElementById('barcode');
            let html5QrCode;
            
            // Initialize scanner when scan button is clicked
            scanButton.addEventListener('click', function() {
                scannerContainer.classList.remove('hidden');
                startScanner();
            });
            
            // Close scanner
            closeScanner.addEventListener('click', function() {
                scannerContainer.classList.add('hidden');
                if (html5QrCode && html5QrCode.isScanning) {
                    html5QrCode.stop();
                }
            });
            
            // Initialize HTML5 QR Code scanner
            function startScanner() {
                html5QrCode = new Html5Qrcode("reader");
                const config = { fps: 10, qrbox: 250 };
                
                html5QrCode.start(
                    { facingMode: "environment" }, // Use rear camera
                    config,
                    onScanSuccess,
                    onScanFailure
                );
            }
            
            // Handle successful scan
            function onScanSuccess(decodedText, decodedResult) {
                // Stop scanning
                html5QrCode.stop();
                
                // Set the barcode input value
                barcodeInput.value = decodedText;
                
                // Hide scanner
                scannerContainer.classList.add('hidden');
                
                // Auto-submit the form
                document.getElementById('barcodeForm').submit();
            }
            
            // Handle scan failure
            function onScanFailure(error) {
                // Just log the error, don't show to user
                console.warn(`QR Code scanning failed: ${error}`);
            }
            
            // Add event listeners to "Select" buttons
            document.querySelectorAll('.select-card-btn').forEach(button => {
                button.addEventListener('click', function() {
                    const cardNumber = this.getAttribute('data-card');
                    barcodeInput.value = cardNumber;
                    // Scroll to the form
                    document.getElementById('barcodeForm').scrollIntoView({ behavior: 'smooth' });
                });
            });
            
            // Auto-focus the barcode input field
            barcodeInput.focus();
        });
    </script>
</body>
</html> 