<?php
// Enable error reporting at the top
ini_set('display_errors', 1);
ini_set('display_startup_errors', 1);
error_reporting(E_ALL);

// Function to debug output
function debug($msg) {
    error_log("[PKPass Debug] " . $msg);
}

// Function to safely remove a directory and its contents
function cleanupDir($dir) {
    if (!is_dir($dir)) {
        return;
    }
    $files = array_diff(scandir($dir), ['.', '..']);
    foreach ($files as $file) {
        $path = $dir . '/' . $file;
        if (is_dir($path)) {
            cleanupDir($path);
        } else {
            unlink($path);
        }
    }
    rmdir($dir);
}

debug("Script location: " . __FILE__);
debug("Document root: " . $_SERVER['DOCUMENT_ROOT']);

require_once '../includes/functions.php';
require_once '../vendor/autoload.php';

use PKPass\PKPass;

// Clear any existing output buffers
while (ob_get_level()) {
    ob_end_clean();
}

// Check if card_number parameter is provided
if (!isset($_GET['card_number'])) {
    header('Content-Type: text/plain');
    die("Card number required");
}

$cardNumber = $_GET['card_number'];
debug("Processing card number: " . $cardNumber);

try {
    // Get card data
$conn = getDbConnection();
$stmt = $conn->prepare("
        SELECT lc.*, c.name, l.name as location_name, l.reward_threshold,
               COALESCE(lc.total_spent, 0) as total_spent, lc.stamp_count
    FROM loyalty_cards lc
    JOIN customers c ON lc.customer_id = c.id
    JOIN locations l ON c.location_id = l.id
        WHERE lc.card_number = ?
");
    $stmt->execute([$cardNumber]);
$card = $stmt->fetch(PDO::FETCH_ASSOC);

if (!$card) {
        header('Content-Type: text/plain');
    die("Invalid card number");
}
    debug("Card data retrieved successfully");

    // Set up pass data
    $data = [
        'formatVersion' => 1,
        'passTypeIdentifier' => 'pass.com.trimscloud.loyalty',
        'serialNumber' => $cardNumber,
        'teamIdentifier' => 'C7CH6P3GLR',
        'organizationName' => $card['location_name'],
        'description' => $card['location_name'] . ' Loyalty Card',
        'logoText' => $card['location_name'],
        'foregroundColor' => 'rgb(67, 97, 238)',
        'backgroundColor' => 'rgb(255, 255, 255)',
        'labelColor' => 'rgb(67, 97, 238)',
        'storeCard' => [
            'primaryFields' => [[
                'key' => 'points',
                'label' => 'STAMPS',
                'value' => $card['stamp_count'] . ' of ' . $card['reward_threshold'],
            ]],
            'secondaryFields' => [[
                'key' => 'name',
                'label' => 'MEMBER',
                'value' => $card['name']
            ]],
            'backFields' => [[
                'key' => 'terms',
                'label' => 'TERMS',
                'value' => "Collect {$card['reward_threshold']} stamps to earn a reward\nOne stamp per purchase"
            ]]
        ],
        'barcode' => [
            'message' => $cardNumber,
            'format' => 'PKBarcodeFormatPDF417',
            'messageEncoding' => 'iso-8859-1'
        ]
    ];

    // Initialize pass
    $certDir = __DIR__ . '/../certs/';
    debug("Certificate directory: " . $certDir);
    
    // Verify certificate files exist
    $certFiles = [
        'signerCert.pem' => $certDir . 'signerCert.pem',
        'signerKey.pem' => $certDir . 'signerKey.pem',
        'wwdr.pem' => $certDir . 'Apple Worldwide Developer Relations Certification Authority.pem'
    ];
    
    foreach ($certFiles as $name => $path) {
        if (!file_exists($path)) {
            throw new Exception("Certificate file not found: " . $name);
        }
        debug("Found certificate: " . $name . " (" . filesize($path) . " bytes)");
    }

    // Create PKPass instance with absolute paths
    $pass = new PKPass(
        realpath($certFiles['signerCert.pem']),
        [realpath($certFiles['signerKey.pem']), 'test'],
        realpath($certFiles['wwdr.pem'])
    );

    // Set pass data
    $pass->setData($data);
    debug("Pass data set successfully");

    // Define required images with their paths
    $imageFiles = [
        'icon.png' => ['required' => true],
        'icon@2x.png' => ['required' => true],
        'logo.png' => ['required' => true],
        'logo@2x.png' => ['required' => true]
    ];
    
    $sourceDir = realpath(__DIR__ . '/../assets/images/wallet_images');
    debug("Source directory: " . $sourceDir);
    
    // Add images to the pass
    foreach ($imageFiles as $file => $config) {
        $sourcePath = $sourceDir . '/' . $file;
        debug("Processing {$file}:");
        debug("  Source: {$sourcePath}");
        
        if (!file_exists($sourcePath)) {
            throw new Exception("Required image not found: " . $file);
        }
        
        if (!is_readable($sourcePath)) {
            throw new Exception("Image file not readable: " . $file);
        }
        
        // Read and add file directly to the pass
        $imageData = file_get_contents($sourcePath);
        if ($imageData === false) {
            throw new Exception("Failed to read image file: " . $file);
        }
        
        if (!$pass->addFile($file, $imageData)) {
            throw new Exception("Failed to add image to pass: " . $file);
        }
        debug("  Added to pass successfully (" . strlen($imageData) . " bytes)");
    }
    
    // Create pass file
    debug("Creating pass...");
    $passFile = $pass->create();
    
    if (!$passFile) {
        throw new Exception("Failed to create pass file");
    }
    debug("Pass created successfully");

    // Set headers for pass download
    header('Pragma: no-cache');
    header('Cache-Control: no-cache');
    header('Content-Type: application/vnd.apple.pkpass');
    header('Content-Disposition: attachment; filename="' . $cardNumber . '.pkpass"');
    header('Content-Length: ' . strlen($passFile));
    
    // Output pass file
    echo $passFile;
    debug("Pass file sent to browser (" . strlen($passFile) . " bytes)");

} catch (Exception $e) {
    // Clear any output buffers
    while (ob_get_level()) {
        ob_end_clean();
    }
    
    header('Content-Type: text/plain');
    $error = "Error creating pass: " . $e->getMessage();
    echo $error . "\n";
    error_log($error);
    debug("Stack trace: " . $e->getTraceAsString());
} 