<?php
// Enable error reporting for debugging
ini_set('display_errors', 1);
ini_set('display_startup_errors', 1);
error_reporting(E_ALL);

// Start logging to a file to track execution
$logFile = __DIR__ . '/../logs/pkpass_debug.log';
// Create logs directory if it doesn't exist
if (!file_exists(dirname($logFile))) {
    mkdir(dirname($logFile), 0755, true);
}
file_put_contents($logFile, "Starting proper PKPass generation: " . date('Y-m-d H:i:s') . "\n", FILE_APPEND);

// Function to log messages
function log_message($message) {
    global $logFile;
    file_put_contents($logFile, $message . "\n", FILE_APPEND);
    return $message;
}

// Check for AJAX request
$isAjax = !empty($_SERVER['HTTP_X_REQUESTED_WITH']) && strtolower($_SERVER['HTTP_X_REQUESTED_WITH']) == 'xmlhttprequest';

require_once __DIR__ . '/../includes/functions.php';
log_message("Loaded functions.php");

require_once __DIR__ . '/pkpass.php';
log_message("Loaded PKPass class");

// Check if card_number parameter is provided
if (!isset($_GET['card_number'])) {
    log_message("Error: Card number parameter is missing");
    if ($isAjax) {
        echo json_encode(['error' => 'Card number parameter is required']);
        exit;
    } else {
        die("Card number parameter is required");
    }
}

$cardNumber = $_GET['card_number'];
log_message("Card Number: $cardNumber");

// Get card details
try {
    $conn = getDbConnection();
    log_message("Database connection established");
    
    $stmt = $conn->prepare("
        SELECT lc.*, c.name, c.email, l.name as location_name, l.logo_url, l.reward_threshold, l.reward_description
        FROM loyalty_cards lc
        JOIN customers c ON lc.customer_id = c.id
        JOIN locations l ON c.location_id = l.id
        WHERE lc.card_number = :card_number
    ");
    $stmt->bindParam(':card_number', $cardNumber);
    $stmt->execute();
    $card = $stmt->fetch(PDO::FETCH_ASSOC);
    
    if (!$card) {
        log_message("Error: Invalid card number - no results found");
        if ($isAjax) {
            echo json_encode(['error' => 'Invalid card number']);
            exit;
        } else {
            die("Invalid card number");
        }
    }
    
    log_message("Card found: " . json_encode($card));
} catch (Exception $e) {
    log_message("Database error: " . $e->getMessage());
    if ($isAjax) {
        echo json_encode(['error' => 'Database error: ' . $e->getMessage()]);
        exit;
    } else {
        die("Database error: " . $e->getMessage());
    }
}

// Certificate information
$certificatePath = __DIR__ . '/../certs/Certificates.p12';
$certificatePassword = 'T@pple9696';
$wwdrCertPath = __DIR__ . '/../certs/AppleWWDRMPCA1G1.cer';

// Check if certificate files exist
if (!file_exists($certificatePath)) {
    log_message("Error: Certificate file not found at: $certificatePath");
    if ($isAjax) {
        echo json_encode(['error' => 'Certificate file not found']);
        exit;
    } else {
        die("Certificate file not found");
    }
}

if (!file_exists($wwdrCertPath)) {
    log_message("Error: WWDR certificate file not found at: $wwdrCertPath");
    if ($isAjax) {
        echo json_encode(['error' => 'WWDR certificate file not found']);
        exit;
    } else {
        die("WWDR certificate file not found");
    }
}

log_message("Certificate files found");

// Define the path to pass images
$passImagesDir = __DIR__ . '/../assets/pass_images';

// Create the pass data
$passData = [
    'formatVersion' => 1,
    'passTypeIdentifier' => 'pass.com.trimscloud.loyaltycard',
    'serialNumber' => $cardNumber,
    'teamIdentifier' => 'C7CH6P3GLR',
    'organizationName' => $card['location_name'],
    'description' => 'Loyalty Card for ' . $card['location_name'],
    'logoText' => $card['location_name'],
    'foregroundColor' => 'rgb(255, 255, 255)',
    'backgroundColor' => 'rgb(67, 97, 238)',
    'labelColor' => 'rgb(255, 255, 255)',
    'webServiceURL' => 'https://dev.trimscloud.com/services/passes/',
    'authenticationToken' => md5($cardNumber . time()),
    'barcodes' => [
        [
            'format' => 'PKBarcodeFormatPDF417',
            'message' => $cardNumber,
            'messageEncoding' => 'iso-8859-1',
            'altText' => $cardNumber
        ]
    ],
    // Include barcode for older iOS versions
    'barcode' => [
        'format' => 'PKBarcodeFormatPDF417',
        'message' => $cardNumber,
        'messageEncoding' => 'iso-8859-1',
        'altText' => $cardNumber
    ],
    'storeCard' => [
        'headerFields' => [
            [
                'key' => 'balance',
                'label' => 'COLLECTED',
                'value' => $card['stamp_count'] . '/' . $card['reward_threshold']
            ]
        ],
        'primaryFields' => [
            [
                'key' => 'offer',
                'label' => 'REWARD',
                'value' => 'Buy ' . ($card['reward_threshold'] - 1) . ' Get 1 FREE!'
            ]
        ],
        'secondaryFields' => [
            [
                'key' => 'name',
                'label' => 'CARDHOLDER',
                'value' => $card['name']
            ]
        ],
        'auxiliaryFields' => [
            [
                'key' => 'stamps',
                'label' => 'STAMPS',
                'value' => $card['stamp_count']
            ]
        ],
        'backFields' => [
            [
                'key' => 'terms',
                'label' => 'TERMS AND CONDITIONS',
                'value' => 'Present this card when making a purchase to collect stamps. Collect ' . $card['reward_threshold'] . ' stamps to get a free item.'
            ],
            [
                'key' => 'website',
                'label' => 'WEBSITE',
                'value' => 'https://dev.trimscloud.com'
            ],
            [
                'key' => 'cardNumber',
                'label' => 'CARD NUMBER',
                'value' => $cardNumber
            ]
        ]
    ]
];

log_message("Pass data prepared");

try {
    // Create a new PKPass instance
    $pass = new PKPass($certificatePath, $certificatePassword, $wwdrCertPath);
    log_message("PKPass instance created");
    
    // Set the pass data
    $pass->setData($passData);
    log_message("Pass data set");
    
    // Set the pass images directory
    $pass->setPassImagesPath($passImagesDir);
    log_message("Pass images path set: $passImagesDir");
    
    // Log pass data for debugging
    log_message("Pass Data JSON: " . json_encode($passData, JSON_PRETTY_PRINT));
    
    // Create the pass
    log_message("Creating PKPass file...");
    $pkpassPath = $pass->create();
    
    if ($pkpassPath) {
        log_message("Pass file created at: $pkpassPath");
        log_message("Pass file size: " . filesize($pkpassPath) . " bytes");
        
        // Verify the file exists and is readable
        if (file_exists($pkpassPath) && is_readable($pkpassPath) && filesize($pkpassPath) > 0) {
            log_message("Pass file verified. Sending to browser...");
            
            // If AJAX, return the path
            if ($isAjax) {
                $relativePath = '/temp/' . basename($pkpassPath);
                echo json_encode(['success' => true, 'file' => $relativePath]);
                exit;
            } else {
                // Send the file to the browser
                $pass->send($pkpassPath);
                // The send() method calls exit(), so the script ends here
            }
        } else {
            $error = "Error: Pass file exists but may not be readable or is empty";
            log_message($error);
            if ($isAjax) {
                echo json_encode(['error' => $error]);
                exit;
            } else {
                die($error);
            }
        }
    } else {
        $error = "Error creating pass: " . $pass->getError();
        log_message($error);
        if ($isAjax) {
            echo json_encode(['error' => $error]);
            exit;
        } else {
            die($error);
        }
    }
} catch (Exception $e) {
    $error = "Exception: " . $e->getMessage();
    log_message($error);
    log_message("Stack trace: " . $e->getTraceAsString());
    if ($isAjax) {
        echo json_encode(['error' => $error]);
        exit;
    } else {
        die("Error: " . $e->getMessage());
    }
}
?> 