<?php
/**
 * PKPass - PHP Class for creating and signing Apple Wallet passes
 * 
 * A simplified class that handles the entire PKPass creation process
 * using only PHP functions, without relying on exec().
 */
class PKPass {
    private $certPath;
    private $certPassword;
    private $wwdrPath;
    private $passData;
    private $passDir;
    private $passImagesDir;
    private $files = [];
    private $tempPath;
    private $lastError;
    
    /**
     * Constructor
     * 
     * @param string $certPath Path to the certificate file (.p12)
     * @param string $certPassword Certificate password
     * @param string $wwdrPath Path to the WWDR certificate file (.cer)
     */
    public function __construct($certPath, $certPassword, $wwdrPath) {
        $this->certPath = $certPath;
        $this->certPassword = $certPassword;
        $this->wwdrPath = $wwdrPath;
        $this->tempPath = dirname(__DIR__) . '/temp';
        
        // Create temp directory if it doesn't exist
        if (!file_exists($this->tempPath)) {
            mkdir($this->tempPath, 0755, true);
        }
        
        // Create a unique pass directory
        $this->passDir = $this->tempPath . '/pass_' . uniqid();
        mkdir($this->passDir, 0755, true);
    }
    
    /**
     * Set pass data (json content)
     * 
     * @param array $data Pass data
     * @return PKPass
     */
    public function setData($data) {
        $this->passData = $data;
        return $this;
    }
    
    /**
     * Set path to pass images directory
     * 
     * @param string $path Path to directory with pass images
     * @return PKPass
     */
    public function setPassImagesPath($path) {
        $this->passImagesDir = $path;
        return $this;
    }
    
    /**
     * Add a file to the pass
     * 
     * @param string $path Path to the file
     * @param string $name Name to use in the pass
     * @return PKPass
     */
    public function addFile($path, $name) {
        if (!file_exists($path)) {
            $this->lastError = "File not found: $path";
            return $this;
        }
        
        $this->files[$name] = $path;
        return $this;
    }
    
    /**
     * Get the last error
     * 
     * @return string Error message
     */
    public function getError() {
        return $this->lastError;
    }
    
    /**
     * Create the PKPass file
     * 
     * @return string|false Path to the created PKPass file or false on failure
     */
    public function create() {
        // 1. Create pass.json
        if (empty($this->passData)) {
            $this->lastError = "No pass data provided";
            return false;
        }
        
        $passJson = json_encode($this->passData);
        if (!$passJson) {
            $this->lastError = "Failed to encode pass data to JSON";
            return false;
        }
        
        file_put_contents($this->passDir . '/pass.json', $passJson);
        
        // 2. Copy pass image files if set
        if (!empty($this->passImagesDir) && is_dir($this->passImagesDir)) {
            // Required images: icon.png, logo.png
            // Optional images: icon@2x.png, logo@2x.png, strip.png, strip@2x.png
            $requiredImages = ['icon.png', 'logo.png'];
            $optionalImages = ['icon@2x.png', 'logo@2x.png', 'strip.png', 'strip@2x.png', 'thumbnail.png', 'thumbnail@2x.png'];
            
            // Copy required images (will throw error if missing)
            foreach ($requiredImages as $image) {
                $imagePath = $this->passImagesDir . '/' . $image;
                if (file_exists($imagePath)) {
                    copy($imagePath, $this->passDir . '/' . $image);
                } else {
                    $this->lastError = "Required pass image missing: $image";
                    return false;
                }
            }
            
            // Copy optional images (will continue if missing)
            foreach ($optionalImages as $image) {
                $imagePath = $this->passImagesDir . '/' . $image;
                if (file_exists($imagePath)) {
                    copy($imagePath, $this->passDir . '/' . $image);
                }
            }
        }
        
        // 3. Copy any additional files
        foreach ($this->files as $fileName => $filePath) {
            copy($filePath, $this->passDir . '/' . $fileName);
        }
        
        // 4. Create the manifest.json
        $manifestEntries = [];
        
        // Get all files in the pass directory
        $files = glob($this->passDir . '/*');
        foreach ($files as $file) {
            $fileName = basename($file);
            
            // Skip the manifest and signature files
            if ($fileName !== 'manifest.json' && $fileName !== 'signature') {
                $manifestEntries[$fileName] = sha1_file($file);
            }
        }
        
        // Write the manifest.json file
        $manifestPath = $this->passDir . '/manifest.json';
        file_put_contents($manifestPath, json_encode($manifestEntries));
        
        // 5. Sign the manifest
        if (!$this->signManifest($manifestPath)) {
            return false;
        }
        
        // 5. Create the .pkpass file (zip)
        $serialNumber = $this->passData['serialNumber'];
        $pkpassPath = $this->tempPath . '/' . $serialNumber . '.pkpass';
        
        if (!$this->createZip($pkpassPath)) {
            return false;
        }
        
        return $pkpassPath;
    }
    
    /**
     * Send the PKPass file to the browser
     * 
     * @param string $pkpassPath Path to the PKPass file
     * @param string $filename Optional filename to use (defaults to the pass type)
     * @return void
     */
    public function send($pkpassPath, $filename = null) {
        if (!file_exists($pkpassPath)) {
            $this->lastError = "PKPass file not found: $pkpassPath";
            return false;
        }
        
        // Get file size
        $fileSize = filesize($pkpassPath);
        if ($fileSize === 0) {
            $this->lastError = "PKPass file is empty";
            return false;
        }
        
        // Clean output buffer
        if (ob_get_level()) {
            ob_end_clean();
        }
        
        // Set the appropriate filename
        if (empty($filename)) {
            // If no filename provided, use the passTypeIdentifier and serialNumber
            $filename = (!empty($this->passData['passTypeIdentifier']) ? 
                        basename($this->passData['passTypeIdentifier']) : 'pass') . 
                        (!empty($this->passData['serialNumber']) ? 
                        '_' . $this->passData['serialNumber'] : '') . '.pkpass';
        }
        
        // Sanitize the filename
        $filename = preg_replace('/[^a-zA-Z0-9_\-\.]/', '', $filename);
        if (empty($filename)) {
            $filename = 'wallet.pkpass';
        }
        
        // Set proper content-type headers
        header('Content-Description: File Transfer');
        header('Content-Type: application/vnd.apple.pkpass');
        header('Content-Disposition: attachment; filename="' . $filename . '"');
        header('Content-Length: ' . $fileSize);
        header('Content-Transfer-Encoding: binary');
        header('Connection: Keep-Alive');
        header('Expires: 0');
        header('Cache-Control: must-revalidate, post-check=0, pre-check=0');
        header('Pragma: public');
        
        // Output file
        readfile($pkpassPath);
        
        // Cleanup
        if (file_exists($this->passDir)) {
            // Delete the pass directory
            $files = glob($this->passDir . '/*');
            foreach ($files as $file) {
                @unlink($file);
            }
            @rmdir($this->passDir);
        }
        
        exit;
    }
    
    /**
     * Sign the manifest file
     * 
     * @param string $manifestPath Path to the manifest.json file
     * @return bool Success status
     */
    private function signManifest($manifestPath) {
        // Get the path for the signature file
        $signaturePath = dirname($manifestPath) . '/signature';
        
        // Read the certificate and key from the p12 file
        $pkcs12 = file_get_contents($this->certPath);
        if (!$pkcs12) {
            $this->lastError = "Could not read certificate file";
            return false;
        }
        
        // Extract the certificate and private key from the p12 file
        if (!openssl_pkcs12_read($pkcs12, $certs, $this->certPassword)) {
            $this->lastError = "Could not read certificate - incorrect password or unsupported format: " . openssl_error_string();
            return false;
        }
        
        // Read the WWDR certificate
        $wwdrCertData = file_get_contents($this->wwdrPath);
        if (!$wwdrCertData) {
            $this->lastError = "Could not read WWDR certificate file";
            return false;
        }
        
        // Create temporary certificate and key files
        $tempDir = $this->passDir;
        $certFile = $tempDir . '/cert.pem';
        $keyFile = $tempDir . '/key.pem';
        $wwdrPemFile = $tempDir . '/wwdr.pem';
        
        // Save the certificate and key to temporary files
        file_put_contents($certFile, $certs['cert']);
        file_put_contents($keyFile, $certs['pkey']);
        
        // Convert WWDR from DER to PEM
        $wwdrPem = $this->derToPem($wwdrCertData);
        file_put_contents($wwdrPemFile, $wwdrPem);
        
        // Create a temporary file for the signature
        $tempSignaturePath = $tempDir . '/signature.tmp';
        
        // Try to sign with openssl_pkcs7_sign using the file:// protocol
        $result = openssl_pkcs7_sign(
            $manifestPath,
            $tempSignaturePath,
            "file://$certFile",
            ["file://$keyFile", $this->certPassword],
            [],
            PKCS7_BINARY | PKCS7_DETACHED,
            "file://$wwdrPemFile"
        );
        
        if (!$result) {
            // First attempt failed, try with direct certificate data
            $result = openssl_pkcs7_sign(
                $manifestPath,
                $tempSignaturePath,
                $certs['cert'],
                [$certs['pkey'], $this->certPassword],
                [],
                PKCS7_BINARY | PKCS7_DETACHED,
                $wwdrPemFile
            );
            
            if (!$result) {
                $error = openssl_error_string();
                $this->lastError = "Failed to sign manifest: $error";
                
                // Cleanup temporary files
                @unlink($certFile);
                @unlink($keyFile);
                @unlink($wwdrPemFile);
                @unlink($tempSignaturePath);
                
                // Return false to indicate failure
                return false;
            }
        }
        
        // Extract the DER signature data
        $signatureContent = file_get_contents($tempSignaturePath);
        if (!$signatureContent) {
            $this->lastError = "Failed to read signature file";
            
            // Cleanup
            @unlink($certFile);
            @unlink($keyFile);
            @unlink($wwdrPemFile);
            @unlink($tempSignaturePath);
            
            return false;
        }
        
        // Extract the signature data (find the actual PKCS7 signature data from PEM format)
        if (preg_match('~Content-Disposition:[^\n]+\s*?([A-Za-z0-9+=/\r\n]+)\s*?-----~s', $signatureContent, $matches)) {
            // Clean up the signature data - remove whitespace
            $signature = preg_replace('/\s+/', '', $matches[1]);
            $derSignature = base64_decode($signature);
            
            if (!$derSignature) {
                $this->lastError = "Failed to decode signature data";
                
                // Cleanup
                @unlink($certFile);
                @unlink($keyFile);
                @unlink($wwdrPemFile);
                @unlink($tempSignaturePath);
                
                return false;
            }
            
            // Write the DER signature
            if (!file_put_contents($signaturePath, $derSignature)) {
                $this->lastError = "Failed to write signature file";
                
                // Cleanup
                @unlink($certFile);
                @unlink($keyFile);
                @unlink($wwdrPemFile);
                @unlink($tempSignaturePath);
                
                return false;
            }
        } else {
            // If we couldn't extract the signature in the normal way,
            // try an alternative approach to get the raw signature data
            
            // Apple Wallet only needs the actual PKCS7 signature, not the full S/MIME format
            // We need to extract just the DER-encoded PKCS7 signature data
            
            // Read the entire signature file
            $fullSignature = file_get_contents($tempSignaturePath);
            
            // Try to find and extract the binary PKCS7 data
            $pos = strpos($fullSignature, "Content-Type: application/x-pkcs7-signature");
            if ($pos !== false) {
                // Find the end of the headers (double newline)
                $pos = strpos($fullSignature, "\r\n\r\n", $pos);
                if ($pos !== false) {
                    $pos += 4; // Move past the double newline
                    $binaryData = substr($fullSignature, $pos);
                    
                    if (!empty($binaryData)) {
                        // Remove any trailing data after the binary content (like PEM footers)
                        $endPos = strpos($binaryData, "\r\n--");
                        if ($endPos !== false) {
                            $binaryData = substr($binaryData, 0, $endPos);
                        }
                        
                        // Try to decode the base64 data
                        $decoded = base64_decode(trim($binaryData));
                        if ($decoded) {
                            // Write this as our signature
                            file_put_contents($signaturePath, $decoded);
                        } else {
                            $this->lastError = "Failed to decode extracted signature data";
                            return false;
                        }
                    } else {
                        $this->lastError = "Extracted signature is empty";
                        return false;
                    }
                } else {
                    $this->lastError = "Failed to find signature data boundaries";
                    return false;
                }
            } else {
                $this->lastError = "Failed to find signature content type in output";
                
                // Cleanup
                @unlink($certFile);
                @unlink($keyFile);
                @unlink($wwdrPemFile);
                @unlink($tempSignaturePath);
                
                return false;
            }
        }
        
        // Cleanup temporary files
        @unlink($certFile);
        @unlink($keyFile);
        @unlink($wwdrPemFile);
        @unlink($tempSignaturePath);
        
        return true;
    }
    
    /**
     * Create a ZIP file with all the pass contents
     * 
     * @param string $pkpassPath Path where the .pkpass file should be saved
     * @return bool Success status
     */
    private function createZip($pkpassPath) {
        // Remove any existing pkpass file
        if (file_exists($pkpassPath)) {
            unlink($pkpassPath);
        }
        
        // Create a new ZIP archive
        $zip = new ZipArchive();
        if ($zip->open($pkpassPath, ZipArchive::CREATE | ZipArchive::OVERWRITE) !== TRUE) {
            $this->lastError = "Could not create ZIP archive";
            return false;
        }
        
        // Add all files from the pass directory to the ZIP
        $files = glob($this->passDir . '/*');
        foreach ($files as $file) {
            $fileName = basename($file);
            $zip->addFile($file, $fileName);
        }
        
        // Close the ZIP
        if (!$zip->close()) {
            $this->lastError = "Could not close ZIP archive";
            return false;
        }
        
        // Verify that the PKPass file exists and has content
        if (!file_exists($pkpassPath) || filesize($pkpassPath) === 0) {
            $this->lastError = "Created PKPass file is empty or missing";
            return false;
        }
        
        return true;
    }
    
    /**
     * Clean up any temporary files
     */
    public function __destruct() {
        // Clean up passDir if it exists
        if (!empty($this->passDir) && file_exists($this->passDir)) {
            // Delete all files in the pass directory
            $files = glob($this->passDir . '/*');
            foreach ($files as $file) {
                @unlink($file);
            }
            
            // Remove the directory
            @rmdir($this->passDir);
        }
    }
    
    /**
     * Convert a DER certificate to PEM format
     * 
     * @param string $derData The binary DER certificate data
     * @return string The certificate in PEM format
     */
    private function derToPem($derData) {
        $pem = chunk_split(base64_encode($derData), 64, "\n");
        return "-----BEGIN CERTIFICATE-----\n" . $pem . "-----END CERTIFICATE-----\n";
    }
    
    /**
     * Extract the DER data from a PEM format
     * 
     * @param string $pemData The PEM format data
     * @return string The binary DER data
     */
    private function pemToDer($pemData) {
        $lines = explode("\n", $pemData);
        $data = '';
        
        $inData = false;
        foreach ($lines as $line) {
            $line = trim($line);
            if ($line === '-----BEGIN CERTIFICATE-----') {
                $inData = true;
            } else if ($line === '-----END CERTIFICATE-----') {
                $inData = false;
            } else if ($inData) {
                $data .= $line;
            }
        }
        
        return base64_decode($data);
    }
}
?> 