<?php
// Enable error reporting
ini_set('display_errors', 1);
ini_set('display_startup_errors', 1);
error_reporting(E_ALL);

echo "<h1>Certificate Test</h1>";

// Certificate paths
$certificatePath = __DIR__ . '/../certs/Certificates.p12';
$wwdrCertPath = __DIR__ . '/../certs/AppleWWDRMPCA1G1.cer';
$certificatePassword = 'T@pple9696';

// Function to convert DER to PEM
function derToPem($derData) {
    $pem = chunk_split(base64_encode($derData), 64, "\n");
    return "-----BEGIN CERTIFICATE-----\n" . $pem . "-----END CERTIFICATE-----\n";
}

// Check file existence
echo "<h2>File Check</h2>";
echo "<p>P12 Certificate: " . ($certificatePath) . " - " . 
     (file_exists($certificatePath) ? "Found (" . filesize($certificatePath) . " bytes)" : "Not Found") . "</p>";
echo "<p>WWDR Certificate: " . ($wwdrCertPath) . " - " . 
     (file_exists($wwdrCertPath) ? "Found (" . filesize($wwdrCertPath) . " bytes)" : "Not Found") . "</p>";

// Test P12 certificate reading
echo "<h2>P12 Certificate Test</h2>";
if (file_exists($certificatePath)) {
    $certData = file_get_contents($certificatePath);
    $result = openssl_pkcs12_read($certData, $certs, $certificatePassword);
    
    if ($result) {
        echo "<p style='color: green;'>✓ P12 Certificate read successfully!</p>";
        echo "<p>Certificate contains:</p>";
        echo "<ul>";
        foreach (array_keys($certs) as $key) {
            echo "<li>" . htmlspecialchars($key) . "</li>";
        }
        echo "</ul>";
    } else {
        echo "<p style='color: red;'>✗ Failed to read P12 certificate: " . openssl_error_string() . "</p>";
    }
} else {
    echo "<p style='color: red;'>P12 Certificate file not found</p>";
}

// Test WWDR certificate reading and conversion
echo "<h2>WWDR Certificate Test</h2>";
if (file_exists($wwdrCertPath)) {
    $wwdrData = file_get_contents($wwdrCertPath);
    
    // Convert to PEM
    $wwdrPem = derToPem($wwdrData);
    
    // Try to read the certificate
    $cert = openssl_x509_read($wwdrPem);
    
    if ($cert) {
        echo "<p style='color: green;'>✓ WWDR Certificate converted and read successfully!</p>";
        
        // Get certificate info
        $certInfo = openssl_x509_parse($cert);
        if ($certInfo) {
            echo "<p>Certificate details:</p>";
            echo "<ul>";
            if (isset($certInfo['subject']['CN'])) {
                echo "<li>Subject: " . htmlspecialchars($certInfo['subject']['CN']) . "</li>";
            }
            if (isset($certInfo['issuer']['CN'])) {
                echo "<li>Issuer: " . htmlspecialchars($certInfo['issuer']['CN']) . "</li>";
            }
            if (isset($certInfo['validTo_time_t'])) {
                echo "<li>Valid Until: " . date('Y-m-d H:i:s', $certInfo['validTo_time_t']) . "</li>";
            }
            echo "</ul>";
        } else {
            echo "<p style='color: red;'>✗ Failed to parse certificate details</p>";
        }
    } else {
        echo "<p style='color: red;'>✗ Failed to read WWDR certificate: " . openssl_error_string() . "</p>";
    }
    
    // Display PEM data
    echo "<h3>PEM Format</h3>";
    echo "<pre>" . htmlspecialchars($wwdrPem) . "</pre>";
} else {
    echo "<p style='color: red;'>WWDR Certificate file not found</p>";
}

// Test signature creation
echo "<h2>Signature Test</h2>";

if (file_exists($certificatePath) && file_exists($wwdrCertPath)) {
    // Create a temporary directory
    $tempDir = __DIR__ . '/../temp/test_cert_' . uniqid();
    if (!file_exists($tempDir)) {
        mkdir($tempDir, 0755, true);
    }
    
    // Create a test manifest.json
    $manifestPath = $tempDir . '/manifest.json';
    $manifest = ['test.json' => sha1('{"test":"data"}')];
    file_put_contents($manifestPath, json_encode($manifest, JSON_PRETTY_PRINT));
    
    // Extract cert and key
    $certData = file_get_contents($certificatePath);
    if (openssl_pkcs12_read($certData, $certs, $certificatePassword)) {
        $certPem = $certs['cert'];
        $keyPem = $certs['pkey'];
        
        // Get WWDR certificate
        $wwdrData = file_get_contents($wwdrCertPath);
        $wwdrPem = derToPem($wwdrData);
        
        // Save temporary certificate files
        $tempWwdrPath = $tempDir . '/wwdr.pem';
        file_put_contents($tempWwdrPath, $wwdrPem);
        
        // Try to sign the manifest
        $signaturePath = $tempDir . '/signature.tmp';
        $result = openssl_pkcs7_sign(
            $manifestPath,
            $signaturePath,
            $certPem,
            [$keyPem, $certificatePassword],
            [],
            PKCS7_BINARY | PKCS7_DETACHED,
            $tempWwdrPath
        );
        
        if ($result) {
            echo "<p style='color: green;'>✓ Manifest signed successfully!</p>";
            
            // Try to extract DER signature
            $signatureContent = file_get_contents($signaturePath);
            
            // Check signature content
            echo "<p>First 100 bytes of signature content:</p>";
            echo "<pre>" . bin2hex(substr($signatureContent, 0, 100)) . "</pre>";
            
            // Find the signature data between the PEM markers
            if (preg_match('~Content-Disposition:[^\n]+\s*?([A-Za-z0-9+=/\r\n]+)\s*?-----~s', $signatureContent, $matches)) {
                $derSignature = base64_decode(preg_replace('/\s+/', '', $matches[1]));
                file_put_contents($tempDir . '/signature', $derSignature);
                echo "<p style='color: green;'>✓ DER signature extracted successfully!</p>";
                
                // Show first bytes of DER signature
                echo "<p>First 100 bytes of DER signature:</p>";
                echo "<pre>" . bin2hex(substr($derSignature, 0, 100)) . "</pre>";
            } else {
                echo "<p style='color: red;'>✗ Failed to extract DER signature from PEM format</p>";
            }
        } else {
            echo "<p style='color: red;'>✗ Failed to sign manifest: " . openssl_error_string() . "</p>";
        }
        
        // Clean up
        @unlink($tempWwdrPath);
        @unlink($signaturePath);
        @unlink($manifestPath);
        @rmdir($tempDir);
    } else {
        echo "<p style='color: red;'>Failed to read certificate for signing test</p>";
    }
} else {
    echo "<p style='color: red;'>One or more certificate files missing</p>";
}

echo "<p><a href='generate_proper_pkpass.php?card_number=LC202503224956'>Generate PKPass</a></p>";
?> 