<?php
/**
 * PHP-Only PKPass Test
 * 
 * This file tests PKPass generation using only PHP functions (no exec() calls)
 * in environments where exec() is disabled.
 */

// Enable error reporting
ini_set('display_errors', 1);
ini_set('display_startup_errors', 1);
error_reporting(E_ALL);

// Header
echo "<h1>PHP-Only PKPass Test</h1>";
echo "<p>This script will test if we can generate a valid PKPass file using only PHP functions without relying on exec().</p>";

// Load required files
echo "<h2>Loading Required Files</h2>";
require_once __DIR__ . '/../includes/functions.php';
echo "<p>✓ Loaded functions.php</p>";

require_once __DIR__ . '/pkpass.php';
echo "<p>✓ Loaded PKPass class</p>";

// Check prerequisites
echo "<h2>Checking Prerequisites</h2>";

// Check temp directory
$tempDir = __DIR__ . '/../temp';
if (!file_exists($tempDir)) {
    mkdir($tempDir, 0755, true);
    echo "<p>✓ Created temporary directory: $tempDir</p>";
} else {
    echo "<p>✓ Temporary directory exists: $tempDir</p>";
}

// Set up logging
$logFile = $tempDir . '/php_only_test.log';
file_put_contents($logFile, "PHP-Only PKPass Test - " . date('Y-m-d H:i:s') . "\n\n");

function log_message($message) {
    global $logFile;
    file_put_contents($logFile, date('Y-m-d H:i:s') . ": " . $message . "\n", FILE_APPEND);
    return $message;
}

echo "<p>✓ Set up logging to: " . basename($logFile) . "</p>";

// Check certificates
$certificatePath = __DIR__ . '/../certs/Certificates.p12';
$certificatePassword = 'T@pple9696';
$wwdrCertPath = __DIR__ . '/../certs/AppleWWDRMPCA1G1.cer';

$certificateStatus = file_exists($certificatePath) ? 
    "Found (" . filesize($certificatePath) . " bytes)" : 
    "Missing";

$wwdrStatus = file_exists($wwdrCertPath) ? 
    "Found (" . filesize($wwdrCertPath) . " bytes)" : 
    "Missing";

echo "<p>P12 Certificate: " . $certificateStatus . "</p>";
echo "<p>WWDR Certificate: " . $wwdrStatus . "</p>";

if (!file_exists($certificatePath) || !file_exists($wwdrCertPath)) {
    echo "<p style='color: red;'>Error: One or more certificate files are missing.</p>";
    exit;
}

// Check if we can read the P12 certificate
echo "<h2>Testing Certificate Reading</h2>";
$pkcs12 = file_get_contents($certificatePath);
if (!openssl_pkcs12_read($pkcs12, $certs, $certificatePassword)) {
    echo "<p style='color: red;'>Error: " . log_message("Failed to read P12 certificate: " . openssl_error_string()) . "</p>";
    exit;
}
echo "<p style='color: green;'>✓ Successfully read P12 certificate</p>";

// Check pass images
echo "<h2>Checking Pass Images</h2>";
$passImagesDir = __DIR__ . '/../assets/pass_images';
$requiredImages = ['icon.png', 'logo.png'];
$missingImages = [];

foreach ($requiredImages as $image) {
    if (!file_exists($passImagesDir . '/' . $image)) {
        $missingImages[] = $image;
    }
}

if (!empty($missingImages)) {
    echo "<p style='color: red;'>Error: Missing required pass images: " . implode(', ', $missingImages) . "</p>";
    exit;
}
echo "<p style='color: green;'>✓ All required pass images are present</p>";

// Create test pass data
echo "<h2>Creating Test Pass</h2>";
echo "<p>Creating pass data...</p>";

$passId = 'TEST' . time();
$passData = [
    'formatVersion' => 1,
    'passTypeIdentifier' => 'pass.com.trimscloud.loyaltycard',
    'serialNumber' => $passId,
    'teamIdentifier' => 'C7CH6P3GLR',
    'organizationName' => 'Test Organization',
    'description' => 'Test Pass',
    'logoText' => 'Test Pass',
    'foregroundColor' => 'rgb(255, 255, 255)',
    'backgroundColor' => 'rgb(60, 60, 60)',
    'labelColor' => 'rgb(255, 255, 255)',
    'barcodes' => [
        [
            'format' => 'PKBarcodeFormatQR',
            'message' => $passId,
            'messageEncoding' => 'iso-8859-1',
            'altText' => $passId
        ]
    ],
    'storeCard' => [
        'headerFields' => [
            [
                'key' => 'header',
                'label' => 'CARD TYPE',
                'value' => 'LOYALTY'
            ]
        ],
        'primaryFields' => [
            [
                'key' => 'name',
                'label' => 'MEMBER',
                'value' => 'Test User'
            ]
        ],
        'secondaryFields' => [
            [
                'key' => 'points',
                'label' => 'POINTS',
                'value' => '100'
            ]
        ],
        'backFields' => [
            [
                'key' => 'terms',
                'label' => 'TERMS AND CONDITIONS',
                'value' => 'This is a test loyalty card for testing PKPass generation.'
            ]
        ]
    ]
];

// Try to create the pass
try {
    echo "<p>Initializing PKPass class...</p>";
    $pass = new PKPass($certificatePath, $certificatePassword, $wwdrCertPath);
    
    echo "<p>Setting pass data...</p>";
    $pass->setData($passData);
    
    echo "<p>Setting pass images directory...</p>";
    $pass->setPassImagesPath($passImagesDir);
    
    echo "<p>Creating PKPass file...</p>";
    log_message("Creating PKPass file...");
    
    $pkpassPath = $pass->create();
    
    if ($pkpassPath && file_exists($pkpassPath)) {
        echo "<p style='color: green;'>✓ Successfully created PKPass file at: " . basename($pkpassPath) . " (" . filesize($pkpassPath) . " bytes)</p>";
        log_message("Successfully created PKPass file: $pkpassPath");
        
        echo "<h2>Success!</h2>";
        echo "<p>The test PKPass file was successfully created using only PHP functions.</p>";
        echo "<p><a href='/temp/" . basename($pkpassPath) . "' style='display: inline-block; background-color: #0071e3; color: white; padding: 10px 20px; text-decoration: none; border-radius: 5px;'>Download Test PKPass</a></p>";
    } else {
        echo "<p style='color: red;'>Error: " . log_message("Failed to create PKPass file: " . $pass->getError()) . "</p>";
    }
} catch (Exception $e) {
    echo "<p style='color: red;'>Exception: " . log_message("Exception while creating PKPass: " . $e->getMessage()) . "</p>";
}

// Display log file link
echo "<h2>Log File</h2>";
echo "<p>For detailed information, check the <a href='/logs/php_only_test.log'>log file</a>.</p>";

// Next steps
echo "<h2>Next Steps</h2>";
echo "<p>If the test was successful, you can now try to generate a real loyalty card PKPass:</p>";
echo "<p><a href='generate_proper_pkpass.php?card_number=LC202503224956' class='px-4 py-2 bg-blue-600 text-white rounded-lg hover:bg-blue-700'>Generate Real Loyalty Card PKPass</a></p>";
?> 