<?php
// Add these lines at the top for debugging
ini_set('display_errors', 1);
ini_set('display_startup_errors', 1);
error_reporting(E_ALL);

require_once '../includes/functions.php';

// Check if customer_id parameter is provided
if (!isset($_GET['customer_id'])) {
    die("Customer ID parameter is required");
}

$customerId = $_GET['customer_id'];
$customer = getCustomerById($customerId);

if (!$customer) {
    die("Invalid customer ID");
}

// If customer is already verified, redirect to card page
if ($customer['is_verified']) {
    $card = getLoyaltyCardByCustomerId($customerId);
    if ($card) {
        $card_url = getJoinUrl('card.php', ['card_number' => $card['card_number']]);
        header("Location: $card_url");
        exit;
    }
}

$error = '';
$success = '';

// Process form submission
if ($_SERVER['REQUEST_METHOD'] === 'POST') {
    $otp = trim($_POST['otp'] ?? '');
    
    if (empty($otp)) {
        $error = "Please enter the verification code";
    } else {
        // Add debugging
        $conn = getDbConnection();
        $stmt = $conn->prepare("
            SELECT * FROM otp_verifications 
            WHERE customer_id = :customer_id 
            ORDER BY created_at DESC 
            LIMIT 1
        ");
        $stmt->bindParam(':customer_id', $customerId);
        $stmt->execute();
        $verification = $stmt->fetch(PDO::FETCH_ASSOC);
        
        if (!$verification) {
            $error = "No verification code found for this customer";
        } else {
            $now = new DateTime();
            $expiryTime = new DateTime($verification['expires_at']);
            
            echo "<pre>";
            echo "Entered OTP: " . $otp . "\n";
            echo "Stored OTP: " . $verification['otp'] . "\n";
            echo "Current time: " . $now->format('Y-m-d H:i:s') . "\n";
            echo "Expiry time: " . $expiryTime->format('Y-m-d H:i:s') . "\n";
            echo "Is expired: " . ($now > $expiryTime ? 'Yes' : 'No') . "\n";
            echo "</pre>";
            
            // Verify OTP
            if (verifyOTP($customerId, $otp)) {
                // Mark customer as verified
                verifyCustomer($customerId);
                
                // Create loyalty card
                $result = createLoyaltyCard($customerId);
                
                if ($result['success']) {
                    // Get the customer's card number
                    $card = getLoyaltyCardByCustomerId($customerId);
                    
                    if ($card) {
                        // Use the getCardUrl function to get the correct URL
                        header("Location: " . getCardUrl($card['card_number']));
                        exit;
                    }
                } else {
                    $error = $result['message'];
                }
            } else {
                $error = "Invalid or expired verification code";
            }
        }
    }
}
?>
<!DOCTYPE html>
<html lang="en">
<head>
    <meta charset="UTF-8">
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <title>Verify Email - Pointify</title>
    <link href="https://cdn.jsdelivr.net/npm/tailwindcss@2.2.19/dist/tailwind.min.css" rel="stylesheet">
</head>
<body class="bg-gray-100 min-h-screen flex items-center justify-center">
    <div class="bg-white p-8 rounded-lg shadow-md w-full max-w-md">
        <h1 class="text-2xl font-bold text-center mb-6">Verify Your Email</h1>
        
        <?php if ($error): ?>
            <div class="bg-red-100 border border-red-400 text-red-700 px-4 py-3 rounded mb-4">
                <?php echo htmlspecialchars($error); ?>
            </div>
        <?php endif; ?>
        
        <p class="mb-4 text-gray-600">
            We've sent a verification code to <strong><?php echo htmlspecialchars($customer['email']); ?></strong>. 
            Please enter the code below to verify your email address.
        </p>
        
        <form method="post" class="space-y-4">
            <div>
                <label for="otp" class="block text-gray-700 mb-2">Verification Code</label>
                <input type="text" id="otp" name="otp" required 
                    class="w-full px-4 py-2 border rounded-lg focus:outline-none focus:ring-2 focus:ring-blue-500"
                    placeholder="Enter 6-digit code">
            </div>
            
            <button type="submit" class="w-full bg-blue-500 text-white py-2 px-4 rounded-lg hover:bg-blue-600 transition duration-200">
                Verify Email
            </button>
        </form>
    </div>
</body>
</html> 