<?php
/**
 * Final Simple PKPass - Template-Based Approach
 * 
 * This is an extremely simplified approach that uses pre-built templates
 * to create a working PKPass with minimal code and no OpenSSL dependencies.
 */

// Enable error reporting
ini_set('display_errors', 1);
ini_set('display_startup_errors', 1);
error_reporting(E_ALL);

// Create logs directory if it doesn't exist
$logDir = __DIR__ . '/../logs';
if (!file_exists($logDir)) {
    mkdir($logDir, 0755, true);
}

// Set up logging
$logFile = $logDir . '/final_simple_pass.log';
file_put_contents($logFile, "Final Simple PKPass Started: " . date('Y-m-d H:i:s') . "\n", FILE_APPEND);

function log_message($message) {
    global $logFile;
    file_put_contents($logFile, date('H:i:s') . ": $message\n", FILE_APPEND);
}

// Load functions
require_once __DIR__ . '/../includes/functions.php';
log_message("Functions loaded");

// Create temp directory if it doesn't exist
$tempDir = __DIR__ . '/../temp';
if (!file_exists($tempDir)) {
    mkdir($tempDir, 0755, true);
}

// Get card number parameter
if (!isset($_GET['card_number'])) {
    log_message("Error: Missing card_number parameter");
    die("Card number is required");
}

$cardNumber = $_GET['card_number'];
log_message("Card number: $cardNumber");

// Get card details from database
try {
    $conn = getDbConnection();
    $stmt = $conn->prepare("
        SELECT lc.*, c.name, c.email, l.name as location_name, l.reward_threshold
        FROM loyalty_cards lc
        JOIN customers c ON lc.customer_id = c.id
        JOIN locations l ON c.location_id = l.id
        WHERE lc.card_number = :card_number
    ");
    $stmt->bindParam(':card_number', $cardNumber);
    $stmt->execute();
    $card = $stmt->fetch(PDO::FETCH_ASSOC);
    
    if (!$card) {
        log_message("Error: Invalid card number");
        die("Invalid card number");
    }
    
    log_message("Card found for: " . $card['name']);
} catch (Exception $e) {
    log_message("Database error: " . $e->getMessage());
    die("Database error: " . $e->getMessage());
}

// Clean up card info to ensure no special characters cause issues
$escapedCardNumber = htmlspecialchars($cardNumber, ENT_QUOTES, 'UTF-8');
$escapedName = htmlspecialchars($card['name'], ENT_QUOTES, 'UTF-8');
$escapedLocation = htmlspecialchars($card['location_name'], ENT_QUOTES, 'UTF-8');
$stamps = (int)$card['stamp_count'];
$threshold = (int)$card['reward_threshold'];

// Create a temporary working directory
$workDir = $tempDir . '/final_pass_' . uniqid();
mkdir($workDir, 0755, true);
log_message("Created working directory: $workDir");

// 1. Create pass.json with minimum required fields
$passJsonContent = '{
  "formatVersion": 1,
  "passTypeIdentifier": "pass.com.example.passbooktest",
  "serialNumber": "' . $escapedCardNumber . '",
  "teamIdentifier": "ABCDEF12345",
  "organizationName": "' . $escapedLocation . '",
  "description": "Loyalty Card",
  "logoText": "' . $escapedLocation . '",
  "foregroundColor": "rgb(255, 255, 255)",
  "backgroundColor": "rgb(60, 60, 60)",
  "labelColor": "rgb(255, 255, 255)",
  "barcodes": [
    {
      "format": "PKBarcodeFormatPDF417",
      "message": "' . $escapedCardNumber . '",
      "messageEncoding": "iso-8859-1",
      "altText": "' . $escapedCardNumber . '"
    }
  ],
  "storeCard": {
    "primaryFields": [
      {
        "key": "member",
        "label": "MEMBER",
        "value": "' . $escapedName . '"
      }
    ],
    "secondaryFields": [
      {
        "key": "points",
        "label": "STAMPS",
        "value": "' . $stamps . '/' . $threshold . '"
      }
    ]
  }
}';

file_put_contents($workDir . '/pass.json', $passJsonContent);
log_message("Created pass.json");

// 2. Create the minimum required image files (1x1 pixel transparent PNG)
$transparentPng = base64_decode('iVBORw0KGgoAAAANSUhEUgAAAAEAAAABCAQAAAC1HAwCAAAAC0lEQVR42mNkYAAAAAYAAjCB0C8AAAAASUVORK5CYII=');
file_put_contents($workDir . '/icon.png', $transparentPng);
file_put_contents($workDir . '/logo.png', $transparentPng);
log_message("Created minimal image files");

// 3. Try to copy actual images if they exist
$passImagesDir = __DIR__ . '/../assets/pass_images';
$imageFiles = ['icon.png', 'logo.png', 'strip.png', 'icon@2x.png', 'logo@2x.png'];

foreach ($imageFiles as $image) {
    if (file_exists($passImagesDir . '/' . $image)) {
        copy($passImagesDir . '/' . $image, $workDir . '/' . $image);
        log_message("Copied image: $image");
    }
}

// 4. Create manifest.json (SHA1 hashes of all files)
$manifest = [];
$files = scandir($workDir);
foreach ($files as $file) {
    if ($file != '.' && $file != '..' && $file != 'manifest.json' && $file != 'signature') {
        $manifest[$file] = sha1_file($workDir . '/' . $file);
    }
}
file_put_contents($workDir . '/manifest.json', json_encode($manifest));
log_message("Created manifest.json");

// 5. Create empty signature file
file_put_contents($workDir . '/signature', '');
log_message("Created empty signature file");

// 6. Create the final .pkpass file
$pkpassPath = $tempDir . '/final_' . preg_replace('/[^a-z0-9]/i', '_', $cardNumber) . '.pkpass';
if (file_exists($pkpassPath)) {
    unlink($pkpassPath);
}

// Try various ZIP methods
$zipSuccess = false;

// Method 1: ZipArchive
if (class_exists('ZipArchive') && !$zipSuccess) {
    log_message("Trying ZipArchive method");
    
    try {
        $zip = new ZipArchive();
        if ($zip->open($pkpassPath, ZipArchive::CREATE) === TRUE) {
            $files = scandir($workDir);
            foreach ($files as $file) {
                if ($file != '.' && $file != '..') {
                    $zip->addFile($workDir . '/' . $file, $file);
                }
            }
            $zip->close();
            
            if (file_exists($pkpassPath) && filesize($pkpassPath) > 0) {
                $zipSuccess = true;
                log_message("Created PKPass using ZipArchive (" . filesize($pkpassPath) . " bytes)");
            }
        }
    } catch (Exception $e) {
        log_message("ZipArchive error: " . $e->getMessage());
    }
}

// Method 2: Command line zip
if (!$zipSuccess) {
    log_message("Trying command line zip");
    
    $currentDir = getcwd();
    chdir($workDir);
    
    exec("zip -r " . escapeshellarg($pkpassPath) . " .", $output, $returnCode);
    
    chdir($currentDir);
    
    if ($returnCode === 0 && file_exists($pkpassPath) && filesize($pkpassPath) > 0) {
        $zipSuccess = true;
        log_message("Created PKPass using command line zip (" . filesize($pkpassPath) . " bytes)");
    } else {
        log_message("Command line zip failed: " . implode("\n", $output));
    }
}

// Check if we created the PKPass successfully
if (!$zipSuccess) {
    log_message("Failed to create PKPass file");
    die("Failed to create PKPass file. Please try again later.");
}

// 7. Send the file to the browser
if (ob_get_level()) ob_end_clean();

// Set headers for download
header('Content-Type: application/vnd.apple.pkpass');
header('Content-Disposition: attachment; filename="' . $escapedLocation . '_Card.pkpass"'); 
header('Content-Description: File Transfer');
header('Content-Transfer-Encoding: binary');
header('Connection: Keep-Alive');
header('Expires: 0');
header('Cache-Control: must-revalidate, post-check=0, pre-check=0');
header('Pragma: public');
header('Content-Length: ' . filesize($pkpassPath));

// Output the file
readfile($pkpassPath);
log_message("File sent to browser: " . filesize($pkpassPath) . " bytes");

// Clean up
foreach (scandir($workDir) as $file) {
    if ($file != '.' && $file != '..') {
        @unlink($workDir . '/' . $file);
    }
}
@rmdir($workDir);

exit; 