<?php
/**
 * Simple PKPass Test
 * 
 * This file provides a simplified test for generating an Apple Wallet PKPass,
 * using only PHP functions without relying on exec().
 */

// Enable error reporting
ini_set('display_errors', 1);
ini_set('display_startup_errors', 1);
error_reporting(E_ALL);

// Create temp directory if it doesn't exist
$tempDir = __DIR__ . '/../temp';
if (!file_exists($tempDir)) {
    mkdir($tempDir, 0755, true);
}

// Set up logging
$logFile = $tempDir . '/pkpass_test.log';
file_put_contents($logFile, "PKPass Test Log - " . date('Y-m-d H:i:s') . "\n\n");

function log_message($message) {
    global $logFile;
    file_put_contents($logFile, date('Y-m-d H:i:s') . ": " . $message . "\n", FILE_APPEND);
    echo htmlspecialchars($message) . "<br>";
}

log_message("Starting PKPass generation test...");

// Check required extensions
$requiredExtensions = ['openssl', 'zip', 'json'];
$missingExtensions = [];

foreach ($requiredExtensions as $ext) {
    if (!extension_loaded($ext)) {
        $missingExtensions[] = $ext;
    }
}

if (!empty($missingExtensions)) {
    log_message("ERROR: Missing required extensions: " . implode(', ', $missingExtensions));
    echo "<p style='color: red;'>Test failed - missing PHP extensions. See log for details.</p>";
    exit;
}

log_message("All required PHP extensions are available");

// Certificate paths
$certificatePath = __DIR__ . '/../certs/Certificates.p12';
$wwdrCertPath = __DIR__ . '/../certs/AppleWWDRMPCA1G1.cer';
$certificatePassword = 'T@pple9696';

// Check certificate files
if (!file_exists($certificatePath)) {
    log_message("ERROR: Certificate file not found: $certificatePath");
    echo "<p style='color: red;'>Test failed - certificate file not found. See log for details.</p>";
    exit;
}

if (!file_exists($wwdrCertPath)) {
    log_message("ERROR: WWDR certificate file not found: $wwdrCertPath");
    echo "<p style='color: red;'>Test failed - WWDR certificate file not found. See log for details.</p>";
    exit;
}

log_message("Certificate files found");

// Test certificate reading
$certData = file_get_contents($certificatePath);
if ($certData) {
    log_message("Certificate file read: " . strlen($certData) . " bytes");
    $certs = [];
    if (openssl_pkcs12_read($certData, $certs, $certificatePassword)) {
        log_message("Certificate successfully read with provided password");
        log_message("Certificate contains: " . implode(", ", array_keys($certs)));
    } else {
        log_message("ERROR: Could not read certificate: " . openssl_error_string());
        echo "<p style='color: red;'>Test failed - could not read certificate. Check password or format. See log for details.</p>";
        
        // Try to get more certificate information
        echo "<div style='background-color: #fff8e1; padding: 15px; border-radius: 5px; margin: 15px 0;'>";
        echo "<h3>Certificate Troubleshooting:</h3>";
        echo "<p>1. Verify that the certificate password is correct: <code>" . htmlspecialchars($certificatePassword) . "</code></p>";
        echo "<p>2. Make sure the certificate is a valid PKCS12 (.p12) file</p>";
        echo "<p>3. If you recreate the certificate, use standard encryption algorithms</p>";
        echo "<p>4. Try running the certificate test: <a href='test_certificates_live.php'>Certificate Test</a></p>";
        echo "</div>";
        
        // Continue for testing other components
    }
} else {
    log_message("ERROR: Could not read certificate file");
    echo "<p style='color: red;'>Test failed - could not read certificate file. See log for details.</p>";
}

// Check if PKPass class exists, if not, include it
if (!class_exists('PKPass')) {
    if (file_exists(__DIR__ . '/pkpass.php')) {
        require_once __DIR__ . '/pkpass.php';
        log_message("Loaded PKPass class from local directory");
    } else {
        log_message("ERROR: PKPass class not found");
        echo "<p style='color: red;'>Test failed - PKPass class not found.</p>";
        exit;
    }
}

// Check for pass images directory
$passImagesDir = __DIR__ . '/../assets/pass_images';
if (!is_dir($passImagesDir)) {
    // Create a pass images directory with test images
    log_message("Pass images directory not found, creating temporary images");
    mkdir($passImagesDir, 0755, true);
    
    // Create minimal required pass images using GD if available
    if (extension_loaded('gd')) {
        // Create icon.png (58x58 pixels)
        $icon = imagecreatetruecolor(58, 58);
        $background = imagecolorallocate($icon, 0, 0, 150);
        $textColor = imagecolorallocate($icon, 255, 255, 255);
        imagefill($icon, 0, 0, $background);
        imagestring($icon, 5, 10, 20, "Test", $textColor);
        imagepng($icon, $passImagesDir . '/icon.png');
        imagedestroy($icon);
        
        // Create logo.png (160x50 pixels)
        $logo = imagecreatetruecolor(160, 50);
        $background = imagecolorallocate($logo, 0, 0, 150);
        $textColor = imagecolorallocate($logo, 255, 255, 255);
        imagefill($logo, 0, 0, $background);
        imagestring($logo, 5, 10, 15, "Test Company", $textColor);
        imagepng($logo, $passImagesDir . '/logo.png');
        imagedestroy($logo);
        
        log_message("Created test images using GD library");
    } else {
        log_message("ERROR: Cannot create test images - GD library not available");
        echo "<p style='color: red;'>Test failed - cannot create test images. GD library not available.</p>";
        exit;
    }
}

// Verify that required images exist
$requiredImages = ['icon.png', 'logo.png'];
$missingImages = [];
foreach ($requiredImages as $image) {
    if (!file_exists($passImagesDir . '/' . $image)) {
        $missingImages[] = $image;
    }
}

if (!empty($missingImages)) {
    log_message("ERROR: Missing required images: " . implode(', ', $missingImages));
    echo "<p style='color: red;'>Test failed - missing required images. See log for details.</p>";
    exit;
}

log_message("Pass images verified");

// Create a test pass JSON
$passData = [
    'formatVersion' => 1,
    'passTypeIdentifier' => 'pass.com.trimscloud.loyaltycard',
    'serialNumber' => 'TEST' . time(),
    'teamIdentifier' => 'C7CH6P3GLR',
    'organizationName' => 'Test Company',
    'description' => 'Test Pass',
    'logoText' => 'Test Pass',
    'foregroundColor' => 'rgb(255, 255, 255)',
    'backgroundColor' => 'rgb(60, 60, 60)',
    'labelColor' => 'rgb(255, 255, 255)',
    'barcodes' => [
        [
            'format' => 'PKBarcodeFormatPDF417',
            'message' => 'Test' . time(),
            'messageEncoding' => 'iso-8859-1',
            'altText' => 'Test Code'
        ]
    ],
    // Include barcode for older iOS versions
    'barcode' => [
        'format' => 'PKBarcodeFormatPDF417',
        'message' => 'Test' . time(),
        'messageEncoding' => 'iso-8859-1',
        'altText' => 'Test Code'
    ],
    'storeCard' => [
        'headerFields' => [
            [
                'key' => 'header',
                'label' => 'CARD TYPE',
                'value' => 'TEST'
            ]
        ],
        'primaryFields' => [
            [
                'key' => 'name',
                'label' => 'MEMBER',
                'value' => 'Test User'
            ]
        ],
        'secondaryFields' => [
            [
                'key' => 'points',
                'label' => 'POINTS',
                'value' => '100'
            ]
        ],
        'backFields' => [
            [
                'key' => 'terms',
                'label' => 'TERMS AND CONDITIONS',
                'value' => 'This is a test loyalty card for testing PKPass generation.'
            ]
        ]
    ]
];

log_message("Pass data prepared");

// Try to create the PKPass
try {
    log_message("Creating PKPass instance...");
    $pass = new PKPass($certificatePath, $certificatePassword, $wwdrCertPath);
    
    log_message("Setting pass data...");
    $pass->setData($passData);
    
    log_message("Setting pass images path...");
    $pass->setPassImagesPath($passImagesDir);
    
    log_message("Creating PKPass file...");
    $pkpassPath = $pass->create();
    
    if ($pkpassPath && file_exists($pkpassPath)) {
        $filesize = filesize($pkpassPath);
        log_message("PKPass file created successfully: " . basename($pkpassPath) . " (" . $filesize . " bytes)");
        
        // Display success message and download link
        echo "<div style='background-color: #e8f5e9; padding: 20px; border-radius: 5px; margin: 20px 0;'>";
        echo "<h2 style='color: #388e3c;'>PKPass Test Successful!</h2>";
        echo "<p>A test PKPass file has been generated with a size of " . $filesize . " bytes.</p>";
        echo "<p><a href='/temp/" . basename($pkpassPath) . "' style='display: inline-block; background-color: #0071e3; color: white; padding: 10px 20px; text-decoration: none; border-radius: 5px;'>Download PKPass File</a></p>";
        echo "</div>";
        
        // Warning about Apple Pass Type Identifier
        echo "<div style='background-color: #fff8e1; padding: 20px; border-radius: 5px; margin: 20px 0;'>";
        echo "<h3 style='color: #ff8f00;'>Important Notes:</h3>";
        echo "<p>For a production pass, you must update the following in the pass.json file:</p>";
        echo "<ul>";
        echo "<li><strong>passTypeIdentifier</strong> - Your Apple Pass Type Identifier</li>";
        echo "<li><strong>teamIdentifier</strong> - Your Apple Developer Team ID</li>";
        echo "<li><strong>Certificates</strong> - Make sure you're using valid Apple Developer Certificates</li>";
        echo "</ul>";
        echo "<p>The test pass may not open in Apple Wallet because it uses a placeholder Apple Pass Type Identifier.</p>";
        echo "</div>";
        
        // Next steps
        echo "<h3>Next Steps:</h3>";
        echo "<p><a href='test_certificates_live.php' style='color: blue;'>Run Comprehensive Certificate Test</a></p>";
        echo "<p><a href='generate_proper_pkpass.php?card_number=LC202503253423' style='color: blue;'>Generate Real Loyalty Card PKPass</a></p>";
    } else {
        log_message("ERROR: Failed to create PKPass file: " . $pass->getError());
        echo "<div style='background-color: #ffebee; padding: 20px; border-radius: 5px; margin: 20px 0;'>";
        echo "<h2 style='color: #c62828;'>PKPass Test Failed</h2>";
        echo "<p>Error: " . htmlspecialchars($pass->getError()) . "</p>";
        echo "<p>See log file for details: " . basename($logFile) . "</p>";
        echo "</div>";
    }
} catch (Exception $e) {
    log_message("EXCEPTION: " . $e->getMessage());
    echo "<div style='background-color: #ffebee; padding: 20px; border-radius: 5px; margin: 20px 0;'>";
    echo "<h2 style='color: #c62828;'>PKPass Test Failed</h2>";
    echo "<p>Exception: " . htmlspecialchars($e->getMessage()) . "</p>";
    echo "<p>See log file for details: " . basename($logFile) . "</p>";
    echo "</div>";
}
?> 